"""suite2p motion utility"""
import glob
import os

from pipeline_util import (
    make_output_directory,
    get_frame_rate_platform_json,
)


def create_input_json(input_dir: str, output_dir: str, plane=None) -> dict:
    """Generates suite2p motion correction input json for a given plane

    Args:
        input_dir (str): path to data directory
        output_dir (str): path to results directory
        plane (int, optional): Plane to process if session directory is passed. Defaults to None.

    Returns:
        dict: suite2p motion correction input json
    """
    if plane is None:
        h5_file = glob.glob(f"{input_dir}/*.h5")[0]
    else:
        h5_file = glob.glob(f"{input_dir}/Other_[0-9]*/Other/ophys/planes/{plane}/*um.h5")[0]
    plane = str(plane)
    output_dir = make_output_directory(output_dir, plane)
    frame_rate_hz = get_frame_rate_platform_json(input_dir)
    return {
        "h5py": h5_file,
        "movie_frame_rate_hz": frame_rate_hz,
        "motion_corrected_output": os.path.join(output_dir, f"{plane}um_suite2p_motion_output.h5"),
        "motion_diagnostics_output": os.path.join(
            output_dir, f"{plane}um_suite2p_rigid_motion_transform.csv"
        ),
        "max_projection_output": os.path.join(
            output_dir, f"{plane}um_suite2p_maximum_projection.png"
        ),
        "avg_projection_output": os.path.join(
            output_dir, f"{plane}um_suite2p_average_projection.png"
        ),
        "registration_summary_output": os.path.join(
            output_dir, f"{plane}um_suite2p_registration_summary.png"
        ),
        "motion_correction_preview_output": os.path.join(
            output_dir, f"{plane}um_suite2p_motion_preview.webm"
        ),
        "output_json": os.path.join(output_dir, f"{plane}um_suite2p_motion_correction_output.json"),
    }



"""Test suite2p motion correction"""
from aind_ophys_pipeline_utils import segmentation_utils
from pipeline_util import get_plane
import pytest


@pytest.mark.parametrize(
    "movie_frame_rate_hz, motion_corrected_output, segmentation_output",
    [
        (30.0, "300um_motion_corrected.h5", "300um_segmentation_output.json"),
    ],
)
def test_create_input_json_capsule(
    movie_frame_rate_hz,
    motion_corrected_output,
    segmentation_output,
    ophys_files_capsule,
    helper_functions,
):
    """Test segmentation input json"""
    movie_fn, base_dir = ophys_files_capsule
    output_dir = helper_functions.create_results_dir("results/")
    plane = get_plane(base_dir)
    motion_corrected_dir = movie_fn.parent
    expected = {
        "suite2p_args": {
            "h5py": str(motion_corrected_dir / motion_corrected_output),
            "tmp_dir": "/scratch",
            "movie_frame_rate_hz": movie_frame_rate_hz,
        },
        "postprocess_args": {
            "motion_corrected_video": str(motion_corrected_dir / motion_corrected_output),
            "output_json": str(output_dir / "300" / segmentation_output),
        },
    }
    actual = segmentation_utils.create_input_json(base_dir, output_dir, plane)
    assert actual == expected



"""Tests dff_utils"""
import pytest

from aind_ophys_pipeline_utils import dff_utils
from pipeline_util import get_plane


@pytest.mark.parametrize(
    "neuropil_trace_h5, dff_h5, movie_frame_rate_hz",
    [
        ("300um_neuropil_trace.h5", "300um_dff.h5", 30.0),
    ],
)
def test_create_input_json_capsule(
    neuropil_trace_h5, dff_h5, movie_frame_rate_hz, ophys_files_capsule, helper_functions
):
    """Test dff input json"""
    movie_fn, base_dir = ophys_files_capsule
    plane = get_plane(base_dir)
    output_dir = helper_functions.create_results_dir("results/")
    expected = {
        "input_file": str(movie_fn.parent / neuropil_trace_h5),
        "movie_frame_rate_hz": movie_frame_rate_hz,
        "output_file": str(output_dir / plane / dff_h5),
    }

    actual = dff_utils.create_input_json(base_dir, output_dir, plane)
    assert actual == expected



"""Tests demixing_utils"""
import pytest
import os

from aind_ophys_pipeline_utils import demixing_utils
from pipeline_util import get_plane, read_h5, remove_roi_nan, write_h5, read_json


@pytest.mark.parametrize(
    "motion_corrected_output, roi_trace_h5, extract_traces_v2_json",
    [
        (
            "300um_motion_corrected.h5",
            "300um_roi_traces.h5",
            "300um_extract_traces_v2.json"
        )
    ],
)
def test_create_input_json_capsule(
    motion_corrected_output,
    roi_trace_h5,
    extract_traces_v2_json,
    ophys_files_capsule,
    helper_functions,
):
    """Test demixing input json"""
    movie_fn, base_dir = ophys_files_capsule
    motion_corrected_fn = movie_fn.parent / motion_corrected_output
    plane = get_plane(base_dir)
    output_dir = helper_functions.create_results_dir("results/")
    os.makedirs("./scratch", exist_ok=True)
    roi_traces = movie_fn.parent / roi_trace_h5
    extract_traces = movie_fn.parent / extract_traces_v2_json
    rois = read_h5(roi_traces, "data", "roi_names")
    roi_data = rois["data"]
    roi_names = rois["roi_names"]
    traces = read_json(extract_traces)
    mod_roi_data, _ = remove_roi_nan(traces, roi_names, roi_data)
    write_h5("./scratch/roi_traces.h5", roi_names=roi_names, data=mod_roi_data)
    expected = {
        "movie_h5": str(motion_corrected_fn),
        "traces_h5": "./scratch/roi_traces.h5",
        "output_file": str(output_dir / plane / f"{plane}um_demixing_output.h5"),
        "output_json": str(output_dir / plane / f"{plane}um_demixing_output.json"),
        "roi_masks": traces,
    }

    actual = demixing_utils.create_input_json(base_dir, output_dir, plane)
    assert actual == expected
